@@ -32,6 +32,7 @@ from ..misc._api import _to_saving_path
 from ..misc._api import _make_saving_path
 from ..misc._api import _fetch_saving_paths
 from ..misc.toolkit import to_2d
+from ..misc.toolkit import inject_mlflow_stuffs
 
 
 def make(
@@ -41,8 +42,9 @@ def make(
     **kwargs: Any,
 ) -> Pipeline:
     kwargs["model"] = model
-    config, increment_config = map(_parse_config, [config, increment_config])
-    return Pipeline.make(update_dict(increment_config, update_dict(config, kwargs)))
+    parsed_config = update_dict(_parse_config(config), kwargs)
+    parsed_increment_config = _parse_config(increment_config)
+    return Pipeline.make(parsed_config, parsed_increment_config)
 
 
 def make_from(
@@ -51,25 +53,46 @@ def make_from(
     kwargs_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
     cuda: Optional[Union[int, str]] = None,
     *,
+    new_model: Optional[str] = None,
+    load_increment: bool = True,
+    increment_config: general_config_type = None,
     model_mapping: Optional[Dict[str, str]] = None,
 ) -> Dict[str, List[Pipeline]]:
     saving_folder = os.path.abspath(saving_folder or "./")
+    if new_model is None:
+        model_mapping = None
+    else:
+        paths_dict = _fetch_saving_paths(identifier, saving_folder)
+        all_paths: List[str] = sum(paths_dict.values(), [])
+        if len(all_paths) > 1:
+            raise ValueError("more than 1 model is detected")
+        pretrain_model = list(paths_dict.keys())[0]
+        model_mapping = {pretrain_model: new_model}
 
     def _core() -> Pipeline:
         compress = os.path.isfile(f"{path}.zip")
         with lock_manager(saving_folder, [path]):
             with Saving.compress_loader(path, compress):
-                kwargs = Saving.load_dict("config", path)
+                config_bundle = Saving.load_dict(Pipeline.config_bundle_name, path)
+        kwargs = config_bundle["config"]
+        if increment_config is None:
+            increment_kwargs = {}
+        else:
+            increment_kwargs = _parse_config(increment_config)
+        if load_increment:
+            loaded_inc_config = config_bundle["increment_config"]
+            increment_kwargs = update_dict(increment_kwargs, loaded_inc_config)
+        increment_kwargs.setdefault("binary_config", config_bundle["binary_config"])
         if model_mapping is not None:
             kwargs["model"] = model_mapping[model]
         if cuda is not None:
-            kwargs["cuda"] = cuda
+            increment_kwargs["cuda"] = cuda
         kwargs.pop("logging_folder", None)
         if kwargs.get("mlflow_config") is not None:
             kwargs["mlflow_config"] = {}
         if kwargs_callback is not None:
             kwargs_callback(kwargs)
-        return make(**kwargs)
+        return make(config=kwargs, increment_config=increment_kwargs)
 
     ms: Dict[str, List[Pipeline]] = {}
     paths_dict = _fetch_saving_paths(identifier, saving_folder)
@@ -85,30 +108,25 @@ def finetune(
     x_cv: data_type = None,
     y_cv: data_type = None,
     *,
-    model: Optional[str] = None,
+    new_model: Optional[str] = None,
     strict: bool = True,
+    load_increment: bool = True,
     identifier: str = "cflearn",
     pretrain_folder: Optional[str] = None,
+    increment_config: general_config_type = None,
     kwargs_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
     state_dict_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
     sample_weights: Optional[np.ndarray] = None,
     cuda: Optional[Union[int, str]] = None,
 ) -> Pipeline:
-    paths_dict = _fetch_saving_paths(identifier, pretrain_folder)
-    all_paths: List[str] = sum(paths_dict.values(), [])
-    if len(all_paths) > 1:
-        raise ValueError("more than 1 model is detected")
-    if model is None:
-        model_mapping = None
-    else:
-        pretrain_model = list(paths_dict.keys())[0]
-        model_mapping = {pretrain_model: model}
     ms = make_from(
         identifier,
         pretrain_folder,
         kwargs_callback,
         cuda,
-        model_mapping=model_mapping,
+        new_model=new_model,
+        load_increment=load_increment,
+        increment_config=increment_config,
     )
     m = list(ms.values())[0][0]
     m.fit(
@@ -347,11 +365,13 @@ def repeat_with(
     if model_configs is None:
         model_configs = {}
 
-    def fetch_config(model_key: str) -> Dict[str, Any]:
+    def fetch_config(i_: int, model_: str) -> Dict[str, Any]:
         local_kwargs = shallow_copy_dict(kwargs)
         assert model_configs is not None
-        local_model_config = model_configs.setdefault(model_key, {})
-        return update_dict(shallow_copy_dict(local_model_config), local_kwargs)
+        local_model_config = model_configs.setdefault(model_, {})
+        fetched = update_dict(shallow_copy_dict(local_model_config), local_kwargs)
+        inject_mlflow_stuffs(model_, config=fetched, run_name=f"run_{i_}")
+        return shallow_copy_dict(fetched)
 
     pipelines_dict: Optional[Dict[str, List[Pipeline]]] = None
     if sequential:
@@ -380,7 +400,7 @@ def repeat_with(
                     leave=False,
                 )
             for i in sub_iterator:
-                local_config = fetch_config(model)
+                local_config = fetch_config(i, model)
                 logging_folder = os.path.join(temp_folder, model, str(i))
                 local_config.setdefault("logging_folder", logging_folder)
                 m = make(model, **shallow_copy_dict(local_config))
@@ -403,13 +423,13 @@ def repeat_with(
         # experiment
         experiment = Experiment(num_jobs=num_jobs)
         for model in models:
-            for _ in range(num_repeat):
-                local_config = fetch_config(model)
+            for i in range(num_repeat):
+                local_config = fetch_config(i, model)
                 experiment.add_task(
                     model=model,
                     compress=compress,
                     root_workplace=temp_folder,
-                    config=shallow_copy_dict(local_config),
+                    config=local_config,
                     data_folder=data_folder,
                 )
         # finalize
