@@ -1,17 +1,14 @@
 import torch
-import pprint
 
 import numpy as np
 
 from typing import *
 from abc import ABCMeta
 from torch import Tensor
-from torch.nn import Module
 from torch.nn import ModuleDict
-from torch.optim import Optimizer
-from cfdata.tabular import ColumnTypes
 from cftool.misc import register_core
-from cftool.misc import LoggingMixin
+from cfdata.tabular import TaskTypes
+from cfdata.tabular import ColumnTypes
 
 try:
     amp: Optional[Any] = torch.cuda.amp
@@ -23,6 +20,8 @@ from ..types import tensor_dict_type
 from ..losses import LossBase
 from ..configs import Configs
 from ..configs import Environment
+from ..protocol import TrainerState
+from ..protocol import ModelProtocol
 from ..protocol import DataLoaderProtocol
 from ..misc.toolkit import to_torch
 from ..modules.heads import HeadBase
@@ -89,7 +88,7 @@ class PipeConfig(NamedTuple):
         return f"{prefix}_{self.head_config}"
 
 
-class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
+class ModelBase(ModelProtocol, metaclass=ABCMeta):
     registered_pipes: Optional[Dict[str, PipeConfig]] = None
 
     def __init__(
@@ -102,15 +101,15 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
     ):
         super().__init__()
         # common
-        self.ema: Optional[EMA] = None
         self.environment = environment
         self.device = environment.device
         self.timing = environment.use_timing_context
-        self.config = environment.model_config
         self.tr_loader = tr_loader
         self.cv_loader = cv_loader
-        self.tr_data = tr_loader.data
+        self.data = self.tr_data = tr_loader.data
         self.cv_data = None if cv_loader is None else cv_loader.data
+        self.num_train = len(self.tr_data)
+        self.num_valid = None if self.cv_data is None else len(self.cv_data)
         self.tr_weights, self.cv_weights = tr_weights, cv_weights
         self._preset_config()
         self._init_config()
@@ -227,6 +226,18 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
             msg = f"attribute '{item}' is not defined in {type(self).__name__}"
             raise AttributeError(msg)
 
+    @property
+    def config(self) -> Dict[str, Any]:
+        return self.environment.model_config
+
+    @property
+    def task_type(self) -> TaskTypes:
+        return self.tr_data.task_type
+
+    @property
+    def labels_key(self) -> str:
+        return self.tr_loader.labels_key
+
     @property
     def num_history(self) -> int:
         num_history = 1
@@ -270,13 +281,13 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
         extractor_unique_key = pipe_config.extractor_unique_key
         extractor_exists = extractor_unique_key in self.extractors
         if pipe_config.reuse_extractor and extractor_exists:
-            extractor = self.extractors[extractor_cfg_key]
+            extractor = self.extractors[extractor_unique_key]
         else:
             if extractor_exists:
                 new_index = 1
                 new_extractor_unique_key = extractor_unique_key
                 while new_extractor_unique_key in self.extractors:
-                    new_extractor_unique_key = f"{extractor_cfg_key}_{new_index}"
+                    new_extractor_unique_key = f"{extractor_unique_key}_{new_index}"
                 extractor_unique_key = new_extractor_unique_key
             if extractor_cfg_key in self._extractor_configs:
                 extractor_config = self._extractor_configs[extractor_cfg_key]
@@ -333,10 +344,12 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
 
     @property
     def input_sample(self) -> tensor_dict_type:
-        x = self.tr_data.processed.x[:2]
-        y = self.tr_data.processed.y[:2]
-        x, y = map(to_torch, [x, y])
-        return {"x_batch": x, "y_batch": y}
+        sample = next(iter(self.tr_loader))
+        if self.tr_loader.return_indices:
+            assert isinstance(sample, tuple)
+            return sample[0]
+        assert isinstance(sample, dict)
+        return sample
 
     @property
     def output_probabilities(self) -> bool:
@@ -363,9 +376,10 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
     def forward(
         self,
         batch: tensor_dict_type,
+        batch_idx: Optional[int] = None,
+        state: Optional[TrainerState] = None,
         batch_indices: Optional[np.ndarray] = None,
         loader_name: Optional[str] = None,
-        batch_step: int = 0,
         **kwargs: Any,
     ) -> tensor_dict_type:
         # batch will have `categorical`, `numerical` and `labels` keys
@@ -376,67 +390,27 @@ class ModelBase(Module, LoggingMixin, metaclass=ABCMeta):
 
     def loss_function(
         self,
+        batch_idx: int,
         batch: tensor_dict_type,
-        batch_indices: np.ndarray,
+        batch_indices: Optional[torch.Tensor],
         forward_results: tensor_dict_type,
-        batch_step: int,
+        state: Optional[TrainerState],
     ) -> tensor_dict_type:
         # requires returning `loss` key
-        y_batch = batch["y_batch"]
+        labels = batch[self.labels_key]
         if self.tr_data.is_clf:
-            y_batch = y_batch.view(-1)
+            labels = labels.view(-1)
         # `sample_weights` could be accessed through:
         # 1) `self.tr_weights[batch_indices]` (for training)
         # 2) `self.cv_weights[batch_indices]` (for validation)
-        losses = self.loss(forward_results, y_batch)
+        losses = self.loss(forward_results, labels)
         return {"loss": losses.mean()}
 
     # API
 
     @property
-    def use_ema(self) -> bool:
-        return self.ema is not None
-
-    def init_ema(self) -> None:
-        ema_decay = self.config.setdefault("ema_decay", 0.0)
-        if 0.0 < ema_decay < 1.0:
-            named_params = list(self.named_parameters())
-            self.ema = EMA(ema_decay, named_params)  # type: ignore
-
-    def apply_ema(self) -> None:
-        if self.ema is None:
-            raise ValueError("`ema` is not defined")
-        self.ema()
-
-    def info(self, *, return_only: bool = False) -> str:
-        msg = "\n".join(["=" * 100, "configurations", "-" * 100, ""])
-        msg += (
-            pprint.pformat(self.environment.config, compact=True)
-            + "\n"
-            + "-" * 100
-            + "\n"
-        )
-        msg += "\n".join(["=" * 100, "parameters", "-" * 100, ""])
-        for name, param in self.named_parameters():
-            if param.requires_grad:
-                msg += name + "\n"
-        msg += "\n".join(["-" * 100, "=" * 100, "buffers", "-" * 100, ""])
-        for name, param in self.named_buffers():
-            msg += name + "\n"
-        msg += "\n".join(
-            ["-" * 100, "=" * 100, "structure", "-" * 100, str(self), "-" * 100, ""]
-        )
-        if not return_only:
-            self.log_block_msg(msg, verbose_level=4)  # type: ignore
-        all_msg, msg = msg, "=" * 100 + "\n"
-        n_tr = len(self.tr_data)
-        n_cv = None if self.cv_data is None else len(self.cv_data)
-        msg += f"{self.info_prefix}training data : {n_tr}\n"
-        msg += f"{self.info_prefix}valid    data : {n_cv}\n"
-        msg += "-" * 100
-        if not return_only:
-            self.log_block_msg(msg, verbose_level=3)  # type: ignore
-        return "\n".join([all_msg, msg])
+    def configurations(self) -> Dict[str, Any]:
+        return self.environment.config
 
     def _split_features(
         self,
