@@ -2,7 +2,11 @@ from typing import *
 from torch.optim import Optimizer
 from torch.optim.lr_scheduler import _LRScheduler
 from torch.optim.lr_scheduler import StepLR
+from torch.optim.lr_scheduler import CyclicLR
+from torch.optim.lr_scheduler import ExponentialLR
+from torch.optim.lr_scheduler import CosineAnnealingLR
 from torch.optim.lr_scheduler import ReduceLROnPlateau
+from torch.optim.lr_scheduler import CosineAnnealingWarmRestarts
 
 from ..misc.toolkit import scheduler_requires_metric
 
@@ -18,6 +22,11 @@ def register_scheduler(name: str) -> Callable[[Type], Type]:
     return _register
 
 
+register_scheduler("cyclic")(CyclicLR)
+register_scheduler("cosine")(CosineAnnealingLR)
+register_scheduler("cosine_restarts")(CosineAnnealingWarmRestarts)
+
+
 @register_scheduler("step")
 class StepLRWithFloor(StepLR):
     def __init__(
@@ -40,6 +49,27 @@ class StepLRWithFloor(StepLR):
         return [max(lr, self.lr_floor) for lr in lrs]
 
 
+@register_scheduler("exponential")
+class ExponentialLRWithFloor(ExponentialLR):
+    def __init__(
+        self,
+        optimizer: Optimizer,
+        gamma: float,
+        last_epoch: int = -1,
+        lr_floor: float = 1.0e-8,
+    ):
+        self.lr_floor = lr_floor
+        super().__init__(optimizer, gamma, last_epoch)
+
+    def get_lr(self) -> List[float]:  # type: ignore
+        lrs = super().get_lr()
+        return [max(lr, self.lr_floor) for lr in lrs]  # type: ignore
+
+    def _get_closed_form_lr(self) -> List[float]:  # type: ignore
+        lrs = super()._get_closed_form_lr()  # type: ignore
+        return [max(lr, self.lr_floor) for lr in lrs]
+
+
 @register_scheduler("plateau")
 class ReduceLROnPlateauWithGet(ReduceLROnPlateau):
     def get_lr(self) -> List[float]:
