@@ -481,7 +481,7 @@ class TrainerState:
     def should_log_lr(self) -> bool:
         if self.log_disabled:
             return False
-        denominator = min(5 * self.num_step_per_epoch, 100)
+        denominator = min(self.num_step_per_epoch, 10)
         return self.step % denominator == 0
 
     @property
@@ -495,6 +495,10 @@ class TrainerState:
         denominator = min(self.num_step_per_epoch, patience)
         return self.step % denominator == 0
 
+    @property
+    def should_log_artifacts(self) -> bool:
+        return self.should_log_metrics_msg
+
     @property
     def should_log_metrics_msg(self) -> bool:
         if self.log_disabled:
