@@ -21,6 +21,7 @@ from cftool.misc import timestamp
 from cftool.misc import update_dict
 from cftool.misc import shallow_copy_dict
 from cftool.misc import fix_float_to_length
+from cftool.misc import lock_manager
 from cftool.misc import timing_context
 from cftool.misc import Incrementer
 from mlflow.exceptions import MlflowException
@@ -396,19 +397,22 @@ class Trainer(MonitoredMixin):
         mlflow_config = environment.mlflow_config
         if mlflow_config is None or self.is_loading or not self.is_rank_0:
             return None
+
         model = self.model.__identifier__
         task_type = self.model.data.task_type.value
         task_name = mlflow_config.setdefault("task_name", f"{model}({task_type})")
         tracking_folder = mlflow_config.setdefault("tracking_folder", os.getcwd())
-        tracking_dir = os.path.abspath(os.path.join(tracking_folder, "mlruns"))
-        os.makedirs(tracking_dir, exist_ok=True)
-        tracking_uri = parse_uri(tracking_dir)
-        self.mlflow_client = mlflow.tracking.MlflowClient(tracking_uri)
-        experiment = self.mlflow_client.get_experiment_by_name(task_name)
-        if experiment is not None:
-            experiment_id = experiment.experiment_id
-        else:
-            experiment_id = self.mlflow_client.create_experiment(task_name)
+        tracking_folder = os.path.abspath(tracking_folder)
+        tracking_dir = os.path.join(tracking_folder, "mlruns")
+        with lock_manager(tracking_folder, ["mlruns"]):
+            os.makedirs(tracking_dir, exist_ok=True)
+            tracking_uri = parse_uri(tracking_dir)
+            self.mlflow_client = mlflow.tracking.MlflowClient(tracking_uri)
+            experiment = self.mlflow_client.get_experiment_by_name(task_name)
+            if experiment is not None:
+                experiment_id = experiment.experiment_id
+            else:
+                experiment_id = self.mlflow_client.create_experiment(task_name)
 
         run = None
         self.from_external = False
@@ -428,12 +432,16 @@ class Trainer(MonitoredMixin):
             run_tags: Dict[str, Any] = mlflow_config.setdefault("run_tags", {})
             run_tags.setdefault(MLFLOW_USER, getpass.getuser())
             run_name = mlflow_config.setdefault("run_name", None)
+            run_name_prefix = mlflow_config.setdefault("run_name_prefix", None)
             if run_name is not None:
+                if run_name_prefix is not None:
+                    run_name = f"{run_name_prefix}_{run_name}"
                 run_tags.setdefault(MLFLOW_RUN_NAME, run_name)
             run = self.mlflow_client.create_run(experiment_id, tags=run_tags)
-
         self.run_id = run.info.run_id
 
+        self.mlflow_params = mlflow_config.get("mlflow_params")
+
     def _prepare_log(self) -> None:
         tuple(
             map(
@@ -447,8 +455,9 @@ class Trainer(MonitoredMixin):
             pass
         if self.mlflow_client is None:
             return None
+        mlflow_params = self.mlflow_params or self.environment.user_defined_config
         if not self.from_external:
-            for key, value in self.environment.user_defined_config.items():
+            for key, value in mlflow_params.items():
                 self.mlflow_client.log_param(self.run_id, key, value)
 
     def _log_scalars(self, metrics: Dict[str, float]) -> None:
@@ -457,6 +466,11 @@ class Trainer(MonitoredMixin):
         for key, value in metrics.items():
             self.mlflow_client.log_metric(self.run_id, key, value, step=self.state.step)
 
+    def _log_artifacts(self) -> None:
+        if self.mlflow_client is None:
+            return None
+        self.mlflow_client.log_artifacts(self.run_id, self.logging_folder)
+
     # deep speed
 
     @property
@@ -526,6 +540,55 @@ class Trainer(MonitoredMixin):
 
     # init
 
+    def default_lr_configs(
+        self,
+        optimizer: Optimizer,
+        optimizer_config: Dict[str, Any],
+    ) -> Dict[str, Dict[str, Any]]:
+        opt_lr = optimizer_config["lr"]
+        # step
+        step_default_cfg = {"step_size": 10 * self.state.num_step_per_epoch}
+        # exponential
+        exp_gamma = (0.1 ** 0.1) ** (1.0 / self.state.num_step_per_epoch)
+        exp_default_cfg = {"gamma": exp_gamma}
+        # cyclic
+        cyclic_default_cfg = {
+            "base_lr": opt_lr,
+            "max_lr": 1.0e-8,
+            "step_size_up": 10 * self.state.num_step_per_epoch,
+            "gamma": exp_gamma,
+        }
+        if "momentum" not in optimizer.defaults:
+            cyclic_default_cfg["cycle_momentum"] = False
+        # cosine
+        cosine_default_cfg = {
+            "eta_min": 1.0e-8,
+            "T_max": 10 * self.state.num_step_per_epoch,
+        }
+        # cosine restarts
+        cosine_restarts_default_cfg = {
+            "eta_min": 1.0e-8,
+            "T_0": 10 * self.state.num_step_per_epoch,
+        }
+        # plateau
+        plateau_default_cfg = {
+            "mode": "max",
+            "min_lr": 1.0e-8,
+            "verbose": self._verbose_level >= 3,
+            "patience": max(
+                10,
+                self.state.snapshot_start_step // self.state.num_step_per_snapshot,
+            ),
+        }
+        return {
+            "step": step_default_cfg,
+            "exponential": exp_default_cfg,
+            "cyclic": cyclic_default_cfg,
+            "cosine": cosine_default_cfg,
+            "cosine_restarts": cosine_restarts_default_cfg,
+            "plateau": plateau_default_cfg,
+        }
+
     def _define_optimizer(
         self,
         params_name: str,
@@ -583,30 +646,24 @@ class Trainer(MonitoredMixin):
             self.config["optimizer_config"] = optimizer_config
             self._optimizer_type = optimizer
             # scheduler
-            step_default_cfg = {"step_size": 10 * self.state.num_step_per_epoch}
-            plateau_default_cfg: Dict[str, Any] = {"mode": "max"}
-            plateau_default_cfg.setdefault("min_lr", 1.0e-8)
-            plateau_default_cfg.setdefault("verbose", self._verbose_level >= 3)
-            plateau_default_cfg.setdefault(
-                "patience",
-                max(
-                    10,
-                    self.state.snapshot_start_step // self.state.num_step_per_snapshot,
-                ),
-            )
-            if scheduler == "step":
-                scheduler_config = update_dict(scheduler_config, step_default_cfg)
-            elif scheduler == "plateau":
-                scheduler_config = update_dict(scheduler_config, plateau_default_cfg)
-            elif scheduler == "warmup":
+            default_lr_configs = self.default_lr_configs(opt, optimizer_config)
+            default_lr_config = default_lr_configs.get(scheduler)
+            error_msg = f"default scheduler config for {scheduler} is not specified"
+            if default_lr_config is not None:
+                scheduler_config = update_dict(scheduler_config, default_lr_config)
+            else:
+                if scheduler != "warmup":
+                    raise ValueError(error_msg)
+            if scheduler == "warmup":
                 sab = scheduler_config.get("scheduler_afterwards_base", "plateau")
+                if sab == "warmup":
+                    raise ValueError("warmup should not be used inside a warmup")
                 sac = scheduler_config.get("scheduler_afterwards_config", {})
-                if sab is not None and isinstance(sab, str):
-                    if sab == "step":
-                        sac = update_dict(sac, step_default_cfg)
-                    elif sab == "plateau":
-                        sac = update_dict(sac, plateau_default_cfg)
-                    sab = scheduler_dict[sab]
+                default_lr_config = default_lr_configs.get(sab)
+                if default_lr_config is None:
+                    raise ValueError(error_msg)
+                sac = update_dict(sac, default_lr_config)
+                sab = scheduler_dict[sab]
                 scheduler_config["scheduler_afterwards_base"] = sab
                 scheduler_config["scheduler_afterwards_config"] = sac
             if scheduler is None:
@@ -807,6 +864,8 @@ class Trainer(MonitoredMixin):
                     self._monitor.plateau_flag = True
 
             with timing_context(self, "monitor.logging", enable=self.timing):
+                if self.state.should_log_artifacts:
+                    self._log_artifacts()
                 if self.state.should_log_metrics_msg:
                     self._log_metrics_msg(self.intermediate)
 
